# Import PyCOMPSs (if installed)
# from exaqute.ExaquteTaskPyCOMPSs import *   # to execute with runcompss

# Import empty definitions (if PyCOMPSs is not installed)
from scheduler.ExaquteTaskLocal import *      # to execute with python3

import sys
sys.dont_write_bytecode = True

import xmc

@ExaquteTask(returns=1)
def abs_Task(value):
    return abs(value)

@ExaquteTask(returns=1)
def returnInput_Task(value):
    return value

@ExaquteTask(returns=1)
def multiplyByScalar_Task(value, scalar):
    return value*scalar

if __name__ == "__main__":

    # RandomGeneratorWrapper
    randomGeneratorInputDict = {'parameters': [2, 6],'generator': 'numpy.random.beta'}

    # SolverWrapper
    solverWrapperInputDict = {'index':[],'parameters':[1,1,.1,1,0.000001,-1]}

    # SampleGenerator
    samplerInputDict = {'randomGenerator': 'xmc.randomGeneratorWrapper.RandomGeneratorWrapper',
                'randomGeneratorInputDict':randomGeneratorInputDict,
                'solverWrapper': 'xmc.classDefs_solverWrapper.KratosSolverWrapper.KratosSolverWrapper',
                'solverWrapperInputDict':solverWrapperInputDict,
                'solverWrapperIndices':None}

    # Moment Estimators
    qoiEstimatorInputDict = {'order':1,'indexSetDimension':1}
    costEstimatorInputDict = {'order':1,'indexSetDimension':0}

    # MonteCarloIndex Constructor
    mci_inputDict = {'indexValue': None,
                'sampler':'xmc.sampleGenerator.SampleGenerator',
                'samplerInputDict':samplerInputDict,
                'qoiEstimator':['xmc.momentEstimator.MomentEstimator'],
                'qoiEstimatorInputDict':[qoiEstimatorInputDict],
                'costEstimator':'xmc.momentEstimator.MomentEstimator',
                'costEstimatorInputDict':costEstimatorInputDict
    }

    ################################# RUN TIME GENERATED ENTITIES END HERE #######################

    # MonoCriterion
    statErrorCriterion = xmc.monoCriterion.MonoCriterion('xmc.methodDefs_monoCriterion.criterionFunctions.isLowerThanOrEqualTo',0.1)

    # MultiCriterion
    criteriaDict = [statErrorCriterion]
    criterionInputs = [['error0']]
    criterion = xmc.multiCriterion.MultiCriterion(criteria=criteriaDict,
                                                  inputsForCriterion=criterionInputs,
                                                  interpreter='xmc.methodDefs_multiCriterion.interpreter.interpretAsStoppingFlag',
                                                  flag='xmc.methodDefs_multiCriterion.flag.plainFlag')

    
    # ErrorEstimator
    MSEErrorEstimator = xmc.errorEstimator.ErrorEstimator(
        error='xmc.methodDefs_errorEstimator.errorEstimation.errorEstimationMSE_Task')

    # ModelEstimator
    inputDictModelEstimator = {'valueForParameters': 
    'xmc.methodDefs_modelEstimator.valueForParameters.geometricModel',
                  'updater': 'xmc.methodDefs_modelEstimator.update.updatePredictorGeometric_Task'}
    biasPredictor = xmc.modelEstimator.ModelEstimator(**inputDictModelEstimator)
    variancePredictor = xmc.modelEstimator.ModelEstimator(**inputDictModelEstimator)
    costPredictor = xmc.modelEstimator.ModelEstimator(**inputDictModelEstimator)

    # HierarchyOptimiser
    inputDict = {'indexSpace': [10],
                 'toleranceSplittingBounds': [.5]*2,
                 'optimalIndexSet': 'xmc.methodDefs_hierarchyOptimiser.optimalIndexSet.incrementLevelsByOne',
                 'optimalSampleNumbers': 'xmc.methodDefs_hierarchyOptimiser.optimalSampleNumbers.multiLevelCostMinimisationMSE',
                 'defaultHierarchy': [[[0],25],[[1],25],[[2],25]]
                }
    hierarchyCostOptimiser = xmc.hierarchyOptimiser.HierarchyOptimiser(**inputDict)
    
    # EstimationAssembler
    expectationAssembler = xmc.estimationAssembler.EstimationAssembler(
        assembleEstimation='xmc.methodDefs_estimationAssembler.assembleEstimation.assembleValue_Task')
    biasAssembler = xmc.estimationAssembler.EstimationAssembler(
        assembleEstimation='xmc.methodDefs_estimationAssembler.assembleEstimation.assembleBias_Task')
    varianceAssembler = xmc.estimationAssembler.EstimationAssembler(
        assembleEstimation='xmc.methodDefs_estimationAssembler.assembleEstimation.assembleStatisticalError_Task')

    # MonteCarloSampler
    inputDict = {'indices': [],
                'indexConstructor':'xmc.monteCarloIndex.MonteCarloIndex',
                'indexConstructorDictionary':mci_inputDict,
                'assemblers': [expectationAssembler,biasAssembler,varianceAssembler],
                'estimatorsForAssembler': [ [[0,[1, True, False]]],[[0,[1, True, False]]],[[0,[1, True, True]]] ],
                'qoiPredictor': [biasPredictor,variancePredictor],
                'estimatorsForPredictor': [ [0,[1, True, False],abs_Task] , [0,[1, True, True],multiplyByScalar_Task] ],
                'costEstimatorsForPredictor': [1, True, False],
                'costPredictor': costPredictor,
                'errorEstimators': [MSEErrorEstimator],
                'assemblersForError': [[1,2]],
                'isCostUpdated':True
            }
    mcSampler = xmc.monteCarloSampler.MonteCarloSampler(**inputDict)

    # XMCAlgorithm
    inputDict = {'monteCarloSampler': mcSampler,
                'hierarchyOptimiser': hierarchyCostOptimiser,
                'stoppingCriterion': criterion,
                'errorsForStoppingCriterion': [0],
                'predictorsForHierarchy': [0,1],
                'costPredictorForHierarchy': [1,True,False],
                'estimatorsForHierarchy': [[0,[1,True,False]],[0,[1,True,True]]],
                'costEstimatorForHierarchy': [1,True,False],
                'tolerancesForHierarchy': [0],
        }

    algo = xmc.XMCAlgorithm(**inputDict)

    algo.runXMC()
