# Import PyCOMPSs (if installed)
# from exaqute.ExaquteTaskPyCOMPSs import *   # to execute with runcompss

# Import empty definitions (if PyCOMPSs is not installed)
from scheduler.ExaquteTaskLocal import *      # to execute with python3

import numpy as np
import xmc.solverWrapper as sw
import time

class MultiLevelRNGSolverWrapper(sw.SolverWrapper):
    """
    solveWrapper type whose solve method accepts a standard normal 
    random number and scales it by an index-dependent mean and 
    variance

    Constructor arguements - 
    solverWrapperIndex - Index-space position of the solverWrapper instance
    rates - list of length 4 whose first and last two entries are respectively
    the constant and rate required to compute the mean and variance using the 
    equation mean (or) variance = constant * 10 ** (-rate*level)
    """
    def __init__(self, **keywordArgs):
        super().__init__(**keywordArgs)
        self.mean = self.parameters[0] * 10**(-(self.parameters[1]*self.solverWrapperIndex[0]))
        self.standardDeviation = self.parameters[2] * 10**(-(self.parameters[3]*self.solverWrapperIndex[0]))
        self.waitForTime = self.parameters[4] * 10**(-(self.parameters[5]*self.solverWrapperIndex[0]))

    @ExaquteTask(returns=2)
    def solveForOneQoI_Task(self,randomInput):
        """
        Returns the value of one QoI, as well as the time taken to compute it
        """
        qoi = None
        start_time = time.time()
        if all([component>=0 for component in self.solverWrapperIndex]):
            qoi = self.mean + self.standardDeviation*randomInput[0]
        else:
            qoi = 0
        time.sleep(self.waitForTime)
        end_time = time.time()
        return qoi, (end_time-start_time)

    def solve(self, randomInput):
        """
        Returns a list of QoI and the time taken to compute them
        """
        number_of_qoi = 1
        qoi,time_for_all_qoi = self.solveForOneQoI_Task(randomInput)
        list_of_qoi = [qoi]*number_of_qoi
        return list_of_qoi,time_for_all_qoi
