import numpy as np
from xmc.tools import packedList

# Import PyCOMPSs (if installed)
# from exaqute.ExaquteTaskPyCOMPSs import *   # to execute with runcompss

# Import empty definitions (if PyCOMPSs is not installed)
from scheduler.ExaquteTaskLocal import *      # to execute with python3

def errorEstimationStatError(*assembledEstimationsList):
    """
    Accept the summation over the variances divided by number of samples over all
    levels and return its square root as the statistical error
    """
    assembledEstimationsList = packedList(assembledEstimationsList)
    assert(len(assembledEstimationsList)==1),"length of assembledEstimationsList passed to errorEstimationStatError is not 1"
    return np.sqrt(assembledEstimationsList[0])

@ExaquteTask(returns=1)
def errorEstimationStatError_Task(*assembledEstimationsList):
    """
    Same as errorEstimationStatError but is a pycompss task
    """
    return errorEstimationStatError(*assembledEstimationsList)

def errorEstimationTerr():
    """
    Return total error given moments/variances
    """
    pass

def errorEstimationMSE(*assembledEstimationsList):
    """
    Return mean squared error given moments/variances
    """
    assembledEstimationsList = packedList(assembledEstimationsList)
    return np.sqrt( assembledEstimationsList[0]**2 + assembledEstimationsList[1] )

@ExaquteTask(returns=1)
def errorEstimationMSE_Task(*assembledEstimationsList):
    """
    Same as errorEstimationMSE but is a PyCOMPSs task
    """
    return errorEstimationMSE(*assembledEstimationsList)
