import numpy as np

def zeroDimensionSamplesOnly(inputDict):
    """
    Return a zero dimensional list of lists, since this is the index space
    for simple Monte Carlo distributions
    """
    return [[]]

def maximalLevelsBiasModel(inputDict):
    """
    Use the bias model to compute the optimal number of levels required by 
    the simulation for a given tolerance and splitting parameter. Return
    the same set of indices if the optimal level is smaller than the existing
    number of levels. 
    """
    oldHierarchy = inputDict['oldHierarchy']
    tolerance = inputDict['tolerances'][0]
    splitting_parameter = inputDict['splittingParameter']
    bias_parameters = inputDict['parametersForModel'][0]

    # Compute level for which bias model predicts 
    optimal_level = np.ceil(-np.log((1-splitting_parameter)*tolerance/bias_parameters[0])/bias_parameters[1])

    # Extend hierarchy until that level is reached, or if optimal_level < max_level, 
    # retain the same hierarchy
    new_levels = [oldHierarchy[i][0] for i in range(len(oldHierarchy))]
    max_level = max([new_levels[i][0] for i in range(len(new_levels))])
    while(max_level < optimal_level):
        new_levels.append([max_level+1])
        # recompute maximum
        max_level = max([new_levels[i][0] for i in range(len(new_levels))])
    return new_levels

def incrementLevelsByOne(inputDict):
    """
    Simply increment the number of levels by one from the current maximum present in the hierarchy
    """
    oldHierarchy = inputDict['oldHierarchy']
    new_levels = [oldHierarchy[i][0] for i in range(len(oldHierarchy))]
    max_level = max([new_levels[i][0] for i in range(len(new_levels))])
    new_levels.append([max_level+1])
    return new_levels

def fullIndexSet(inputDict):
    """
    Generate full index set
    """
    pass

def optimalIndexSet(inputDict):
    """
    Generate cost optimal index set
    """
    pass
