# Import PyCOMPSs (if installed)
# from exaqute.ExaquteTaskPyCOMPSs import *   # to execute with runcompss

# Import empty definitions (if PyCOMPSs is not installed)
from scheduler.ExaquteTaskLocal import *      # to execute with python3

def centralMomentTaskWrapper(dimension,order,*args):
    if dimension==0:
        if order==1:
            return computeCentralMomentsOrderOneDimensionZero_Task(*args)
        elif order==2:
            return computeCentralMomentsOrderTwoDimensionZero_Task(*args)
        elif order==3:
            return computeCentralMomentsOrderThreeDimensionZero_Task(*args)
        elif order==4:
            return computeCentralMomentsOrderFourDimensionZero_Task(*args)
        else:
            raise ValueError('Moments of order > 4 are not supported yet for dimension 0.')
    elif dimension==1:
        if order==1:
            return computeCentralMomentsOrderOneDimensionOne_Task(*args)
        else:
            raise ValueError('Moments of order > 0 are not supported yet for dimension 1.')
    else:
        raise ValueError('Index sets of dimension > 0 are not supported yet.')



@ExaquteTask(returns=1)
def computeCentralMomentsOrderOneDimensionZero_Task(power_sum_1,number_samples):
    central_moment_1 = power_sum_1/number_samples
    return central_moment_1

@ExaquteTask(returns=1)
def computeCentralMomentsOrderTwoDimensionZero_Task(power_sum_1,power_sum_2,number_samples):
    central_moment_2 = (number_samples*power_sum_2-power_sum_1**2) / ((number_samples-1)*number_samples)
    return central_moment_2

@ExaquteTask(returns=1)
def computeCentralMomentsOrderThreeDimensionZero_Task(power_sum_1,power_sum_2,power_sum_3,number_samples):
    central_moment_3 = (number_samples**2*power_sum_3-3*number_samples*power_sum_2*power_sum_1+2*power_sum_1**3) / \
    ((number_samples-2)*(number_samples-1)*number_samples)
    return central_moment_3

@ExaquteTask(returns=1)
def computeCentralMomentsOrderFourDimensionZero_Task(power_sum_1,power_sum_2,power_sum_3,power_sum_4,number_samples):
    central_moment_4 = ((-4*number_samples**2+8*number_samples-12)*power_sum_3*power_sum_1+ \
    (number_samples**3-2*number_samples**2+3*number_samples)*power_sum_4+ \
    6*number_samples*power_sum_2*power_sum_1**2+(9-6*number_samples)*power_sum_2**2-3*power_sum_1**4) / \
    ((number_samples-3)*(number_samples-2)*(number_samples-1)*number_samples)
    return central_moment_4

@ExaquteTask(returns=1)
def computeCentralMomentsOrderOneDimensionOne_Task(power_sum_10,power_sum_01,number_samples):
    partial_central_moment_1 = power_sum_01/number_samples
    return partial_central_moment_1
