import numpy as np

# Import PyCOMPSs (if installed)
# from exaqute.ExaquteTaskPyCOMPSs import *   # to execute with runcompss

# Import empty definitions (if PyCOMPSs is not installed)
from scheduler.ExaquteTaskLocal import *      # to execute with python3

def centralMomentErrorTaskWrapper(dimension,order,*args):
    if dimension==0:
        if order==1:
            return computeCentralMomentsErrorEstimationOrderOneDimensionZero_Task(*args)
        elif order==2:
            return computeCentralMomentsErrorEstimationOrderTwoDimensionZero_Task(*args)
        else:
            raise ValueError('Moments of order > 2 are not supported yet for dimension 0.')
    elif dimension==1:
        if order==1:
            return computeCentralMomentsErrorEstimationOrderOneDimensionOne_Task(*args)
        else:
            raise ValueError('Moments of order > 1 are not supported yet for dimension 1.')
    else:
        raise ValueError('Index set of dimension > 2 are not supported yet.')

@ExaquteTask(returns=1)
def computeCentralMomentsErrorEstimationOrderOneDimensionZero_Task(power_sum_1,power_sum_2,number_samples):
    central_moment_2 = (number_samples*power_sum_2-power_sum_1**2) / ((number_samples-1)*number_samples)
    estimation_error_1 = central_moment_2/number_samples
    return estimation_error_1

@ExaquteTask(returns=1)
def computeCentralMomentsErrorEstimationOrderTwoDimensionZero_Task(power_sum_1,power_sum_2,power_sum_3,power_sum_4,number_samples):
    central_moment_2 = (number_samples*power_sum_2-power_sum_1**2) / ((number_samples-1)*number_samples)
    central_moment_4 = ((-4*number_samples**2+8*number_samples-12)*power_sum_3*power_sum_1+ \
    (number_samples**3-2*number_samples**2+3*number_samples)*power_sum_4+ \
    6*number_samples*power_sum_2*power_sum_1**2+(9-6*number_samples)*power_sum_2**2-3*power_sum_1**4) / \
    ((number_samples-3)*(number_samples-2)*(number_samples-1)*number_samples)
    estimation_error_2 = central_moment_4/number_samples - \
        (central_moment_2**2)*(number_samples-3)/(number_samples*(number_samples-1))
    return estimation_error_2

@ExaquteTask(returns=1)
def computeCentralMomentsErrorEstimationOrderOneDimensionOne_Task(power_sum_10,power_sum_01,power_sum_20,power_sum_11,power_sum_02,number_samples):
    partial_central_moment_2 = (number_samples*power_sum_02-power_sum_01**2) / ((number_samples-1)*number_samples)
    partial_estimation_error_1 = partial_central_moment_2/number_samples
    return partial_estimation_error_1
