from xmc.multiCriterion import MultiCriterion
interpretationStructure =  MultiCriterion.flagStructure

def interpretAsStoppingFlag(flag):
    """
    This is the trivial interpreter for an input which is a boolean answering 
    the question `Must I stop the algorithm now'?
    """
    interpretation = interpretationStructure()
    interpretation['stop'] = flag[0]
    return interpretation

def interpretAsConvergenceAndIterationBounds(flags):
    """
    This is the common interpreter which expects three booleans as input:
    1. Is convergence achieved?
    2. Is the number of iteration high enough?
    3. Is the number of iteration low enough?
    N.B. 2 and 3 are actually interchangeable.
    """
    flag = flags[1] or (flags[0] and flags[2])
    interpretation = interpretAsStoppingFlag(flag)
    return interpretation

def interpretAsMultipleRequiredConvergencesAndIterationBounds(flags):
    """
    This is the common interpreter which expects N booleans as input:
    + 1 to N-2: convergence flags, all necessary (e.g. split tolerance)
    + N-1: is the number of iteration high enough?
    + N: is the number of iteration low enough?
    """
    flags = [all(flags[0:-2]), flags[-2], flags[-1]]
    interpretation = interpretAsConvergenceAndIterationBounds(flags)
    return interpretation

def interpretAsMultipleAlternativeConvergencesAndIterationBounds(flags):
    """
    This is the common interpreter which expects N booleans as input:
    + 1 to N-2: alternative convergence flags (at least one necessary)
    + N-1: is the number of iteration high enough?
    + N: is the number of iteration low enough?
    """
    flags = [any(flags[0:-2]), flags[-2], flags[-1]]
    interpretation = interpretAsConvergenceAndIterationBounds(flags)
    return interpretation

